package es.uca.xmleye.xpath;

/*
 Copyright (C) 2005-2008 Antonio García Domínguez

 This program is free software; you can redistribute it 
 and/or modify it under the terms of the GNU General Public 
 License as published by the Free Software Foundation; 
 either version 2 of the License, or (at your option) any 
 later version.

 This program is distributed in the hope that it will be 
 useful, but WITHOUT ANY WARRANTY; without even the implied 
 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE. See the GNU General Public License for more 
 details.

 You should have received a copy of the GNU General Public 
 License along with this program; if not, write to the Free 
 Software Foundation, Inc., 59 Temple Place, Suite 330, 
 Boston, MA 02111-1307 USA
 */

import java.util.*;
import org.w3c.dom.*;

/**
 * Clase de conveniencia, que nos permite combinar fácilmente varias NodeList o
 * invertirlas. Esto es útil, por ejemplo, para combinar resultados de varias
 * búsquedas.
 */
public class VectorNodeList extends ArrayList<Node> implements NodeList {
	/**
	 * 
	 */
	private static final long serialVersionUID = -4462144690826849047L;

	/**
	 * Crea un VectorNodeList vacío.
	 */
	public VectorNodeList() {
		super();
	}

	/**
	 * Crea un VectorNodeList a partir de una lista de nodos ya existente.
	 * 
	 * @param l
	 *            Lista cuyos elementos van a copiarse.
	 */
	public VectorNodeList(NodeList l) {
		super();

		for (int i = 0; i < l.getLength(); ++i) {
			add(l.item(i));
		}
	}

	/**
	 * Copia los elementos de otra lista de nodos en ésta, añadiéndolos al
	 * final.
	 * 
	 * @param l
	 *            Lista con los nodos.
	 */
	public void add(NodeList l) {
		for (int i = 0; i < l.getLength(); ++i) {
			add(l.item(i));
		}
	}

	/**
	 * Retira todos los elementos contenidos en esta lista que pertenezcan
	 * también a <code>l</code>. Se considera que dos nodos n1 y n2 son
	 * iguales si <code>(n1 == null ? n2 == null : n1.equals(n2))</code>.
	 * 
	 * @param l
	 *            Lista con los nodos.
	 */
	public void remove(NodeList l) {
		for (int i = 0; i < l.getLength(); ++i) {
			remove(l.item(i));
		}
	}

	/**
	 * Invierte la lista in-situ.
	 */
	public void invertir() {
		int n = size();

		for (int i = 0; i < n / 2; ++i) {
			Node a = get(i);
			Node b = get(n - i - 1);

			set(n - i - 1, a);
			set(i, b);
		}
	}

	/* -------------------- Interfaz URIResolver -------------------- */

	public Node item(int i) {
		return get(i);
	}

	public int getLength() {
		return size();
	}
}
