package es.uca.xmleye.gui.trees;

/*
  Copyright (C) 2005-2008 Antonio García Domínguez
  
  This program is free software; you can redistribute it 
  and/or modify it under the terms of the GNU General Public 
  License as published by the Free Software Foundation; 
  either version 2 of the License, or (at your option) any 
  later version.
  
  This program is distributed in the hope that it will be 
  useful, but WITHOUT ANY WARRANTY; without even the implied 
  warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
  PURPOSE. See the GNU General Public License for more 
  details.
  
  You should have received a copy of the GNU General Public 
  License along with this program; if not, write to the Free 
  Software Foundation, Inc., 59 Temple Place, Suite 330, 
  Boston, MA 02111-1307 USA
*/

import java.io.*;
import java.util.LinkedList;
import java.util.NoSuchElementException;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.w3c.dom.*;

import es.uca.xmleye.gui.trees.DOMTreeModel;

import junit.framework.*;
import junit.textui.*;

/** 
    Pruebas de unidad sobre DOMTreeModel.
*/
public class DOMTreeModelTest extends TestCase 
{
  private Document _docXML;
  private DOMTreeModel _domTree;

  /** Crea el caso de prueba. */
  public DOMTreeModelTest(String nombre) 
  {
    super(nombre);
  }
  
  /** 
      <p>Prueba de estructura del modelo. Debe cumplirse:</p>

      <ul>
        <li>el documento original es devuelto por getDocumento()</li>
        <li>la raíz del árbol es el Element de mayor nivel</li>
        <li>un nodo está en el modelo si y sólo si está en el documento y
        es un elemento</li>
      </ul>
  */
  public void testEstructuraArbol() 
  {
    assertTrue( _docXML.equals(_domTree.getDocumento() ) );
    assertTrue( _docXML.getDocumentElement().getTagName() 
		== ((Element)_domTree.getRoot()).getTagName() );

    // El recorrido del arbol DOM y el modelo de este debe ser el
    // mismo, limitándose a los nodos de tipo elemento.
    LinkedList<Node> pilaDoc = new LinkedList<Node>();
    LinkedList<Node> pilaMod = new LinkedList<Node>();
    pilaDoc.addLast(_docXML.getDocumentElement());
    pilaMod.addLast((Node)_domTree.getRoot());

    try {
      while (!pilaDoc.isEmpty() && !pilaMod.isEmpty()) {
	Node nodoDom = pilaDoc.removeLast();
	Node nodoMod = pilaMod.removeLast();
	
	if (!nodoDom.equals(nodoMod)) {
	  fail("Recorridos no coinciden.");
	}

	NodeList nodos = nodoDom.getChildNodes();
	for (int i=0; i<nodos.getLength(); ++i) {
	  if (nodos.item(i).getNodeType() == Node.ELEMENT_NODE) {
	    pilaDoc.addLast(nodos.item(i));
	  }
	}
	
	for (int i=0; i<_domTree.getChildCount(nodoMod); ++i) {
	  pilaMod.addLast((Node)_domTree.getChild(nodoMod,i));
	}
      }
    } catch (NoSuchElementException nEx) {
      fail("Faltan nodos en uno de los recorridos.");
    }

    if (!pilaDoc.isEmpty() || !pilaMod.isEmpty()) {
      fail("Sobran nodos (o no son del tipo correcto) en un recorrido.");
    }

  }

  /** Crea las pruebas para que JUnit las ejecute. */
  public static Test suite() 
  {
    return new TestSuite(DOMTreeModelTest.class);
  }

  /** Lanza las pruebas de unidad. */
  public static void main(String[] args) 
  {
    TestRunner.run(suite());
  }

  /** Preparación: carga un documento XML para pruebas. */
  protected void setUp() throws ParserConfigurationException, SAXException,
                                FileNotFoundException, IOException
  {
    DocumentBuilder dBuilder = 
	DocumentBuilderFactory.newInstance().newDocumentBuilder();
    
    final FileInputStream fIS = 
	new FileInputStream("build/es/uca/xmleye/gui/trees/testTree.xml");
    _docXML  = dBuilder.parse(fIS);
    _domTree = new DOMTreeModel(_docXML);
  }

  /** Realiza la limpieza. */
  protected void tearDown() {
    _domTree = null;
  }

}
